@@ -1,8 +1,6 @@
 setMethod(".IC",
           signature(x = "REBMIX"),
-function(x, 
-  Criterion = "AIC",
-  pos = 1, ...)
+function(x, Criterion, pos, ...)
 {
   if (missing(x)) {
     stop(sQuote("x"), " object of class REBMIX is requested!", call. = FALSE)
@@ -56,11 +54,10 @@ function(x,
   if (C == .rebmix$Preprocessing[1]) {
     y0 <- as.double(x@summary[pos, paste("y0", if (d > 1) 1:d else "", sep = "")])
 
-    output <- .C("RInformationCriterionH",
+    output <- .C("RInformationCriterionHMIX",
       h = as.double(h),
       y0 = as.double(y0),
       k = as.integer(x@summary[pos, "v/k"]),
-      d = as.integer(d),
       Criterion = as.character(Criterion),
       c = as.integer(c),
       w = as.double(x@w[[pos]]),
@@ -84,9 +81,8 @@ function(x,
   } 
   else 
   if (C == .rebmix$Preprocessing[2]) {
-    output <- .C("RInformationCriterionPW",
+    output <- .C("RInformationCriterionPWMIX",
       h = as.double(h),
-      d = as.integer(d),
       Criterion = as.character(Criterion),
       c = as.integer(c),
       w = as.double(x@w[[pos]]),
@@ -112,10 +108,9 @@ function(x,
   if (C == .rebmix$Preprocessing[3]) {
     k <- as.integer(x@summary[pos, "v/k"]) 
 
-    output <- .C("RInformationCriterionKNN",
+    output <- .C("RInformationCriterionKNNMIX",
       h = as.double(h),
       k = as.integer(x@summary[pos, "v/k"]),
-      d = as.integer(d),
       Criterion = as.character(Criterion),
       c = as.integer(c),
       w = as.double(x@w[[pos]]),
@@ -145,16 +140,147 @@ function(x,
 
 setMethod(".IC",
           signature(x = "REBMVNORM"),
-function(x, 
-  Criterion = "AIC",
-  pos = 1, ...)
+function(x, Criterion, pos, ...)
 {
+  if (missing(x)) {
+    stop(sQuote("x"), " object of class REBMVNORM is requested!", call. = FALSE)
+  }
+  
+  if (!is.character(Criterion)) {
+    stop(sQuote("Criterion"), " character vector is requested!", call. = FALSE)
+  }
+
+  Criterion <- match.arg(Criterion, .rebmix$Criterion, several.ok = TRUE)  
+
+  if (!is.wholenumber(pos)) {
+    stop(sQuote("pos"), " integer is requested!", call. = FALSE)
+  }
+  
+  length(pos) <- 1
+
+  if ((pos < 1) || (pos > nrow(x@summary))) {
+    stop(sQuote("pos"), " must be greater than 0 and less or equal than ", nrow(x@summary), "!", call. = FALSE)
+  }
+
+  Dataset <- as.character(x@summary[pos, "Dataset"])
+
+  X <- as.matrix(x@Dataset[[which(names(x@Dataset) == x@summary[pos, "Dataset"])]])
+
+  n <- nrow(X)
+  d <- ncol(X)
+
+  h <- as.double(x@summary[pos, paste("h", if (d > 1) 1:d else "", sep = "")])
+  
+  Names <- names(x@Theta[[pos]])
+  
+  c <- length(x@w[[pos]])
+
+  pdf <- unlist(x@Theta[[pos]][grep("pdf", Names)])
+  
+  pdf <- match.arg(pdf, .rebmix$pdf, several.ok = TRUE)
+  
+  theta1 <- unlist(x@Theta[[pos]][grep("theta1", Names)])
+  
+  theta1[is.na(theta1)] <- 0
+
+  theta2 <- unlist(x@Theta[[pos]][grep("theta2", Names)])
+  
+  theta2[is.na(theta2)] <- 0
+
+  length(pdf) <- d
+
+  C <- x@summary[pos, "Preprocessing"]
+
+  if (C == .rebmix$Preprocessing[1]) {
+    y0 <- as.double(x@summary[pos, paste("y0", if (d > 1) 1:d else "", sep = "")])
+
+    output <- .C("RInformationCriterionHMVNORM",
+      h = as.double(h),
+      y0 = as.double(y0),
+      k = as.integer(x@summary[pos, "v/k"]),
+      Criterion = as.character(Criterion),
+      c = as.integer(c),
+      w = as.double(x@w[[pos]]),
+      length.pdf = as.integer(d),
+      length.Theta = as.integer(4),
+      length.theta = as.integer(c(d, d * d, -d * d, -1)),      
+      pdf = as.character(pdf),
+      Theta = as.double(c(theta1, theta2)),
+      n = as.integer(n),
+      x = as.double(X),
+      IC = double(1),
+      logL = double(1),
+      M = integer(1),
+      D = double(1),
+      error = integer(1),
+      PACKAGE = "rebmix")
+
+    if (output$error == 1) {
+      stop("in .IC!", call. = FALSE); return(NA)
+    }
+  } 
+  else 
+  if (C == .rebmix$Preprocessing[2]) {
+    output <- .C("RInformationCriterionPWMVNORM",
+      h = as.double(h),
+      Criterion = as.character(Criterion),
+      c = as.integer(c),
+      w = as.double(x@w[[pos]]),
+      length.pdf = as.integer(d),
+      length.Theta = as.integer(4),
+      length.theta = as.integer(c(d, d * d, -d * d, -1)),
+      pdf = as.character(pdf),
+      Theta = as.double(c(theta1, theta2)),
+      n = as.integer(n),
+      x = as.double(X),
+      IC = double(1),
+      logL = double(1),
+      M = integer(1),
+      D = double(1),
+      error = integer(1),
+      PACKAGE = "rebmix")
+
+    if (output$error == 1) {
+      stop("in .IC!", call. = FALSE); return(NA)
+    }
+  } 
+  else
+  if (C == .rebmix$Preprocessing[3]) {
+    k <- as.integer(x@summary[pos, "v/k"]) 
+
+    output <- .C("RInformationCriterionKNNMVNORM",
+      h = as.double(h),
+      k = as.integer(x@summary[pos, "v/k"]),
+      Criterion = as.character(Criterion),
+      c = as.integer(c),
+      w = as.double(x@w[[pos]]),
+      length.pdf = as.integer(d),
+      length.Theta = as.integer(4),
+      length.theta = as.integer(c(d, d * d, -d * d, -1)),
+      pdf = as.character(pdf),
+      Theta = as.double(c(theta1, theta2)),
+      n = as.integer(n),
+      x = as.double(X),
+      IC = double(1),
+      logL = double(1),
+      M = integer(1),
+      D = double(1),
+      error = integer(1),
+      PACKAGE = "rebmix")
+
+    if (output$error == 1) {
+      stop("in .IC!", call. = FALSE); return(NA)
+    }
+  }
+  
+  rm(list = ls()[!(ls() %in% c("output"))])
+
+  invisible(output)
 }) ## .IC
 
 setMethod("logL",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -171,8 +297,7 @@ function(x,
 
 setMethod("AIC",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -189,8 +314,7 @@ function(x,
 
 setMethod("AIC3",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -207,8 +331,7 @@ function(x,
 
 setMethod("AIC4",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -225,8 +348,7 @@ function(x,
 
 setMethod("AICc",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -243,8 +365,7 @@ function(x,
 
 setMethod("BIC",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -261,8 +382,7 @@ function(x,
 
 setMethod("CAIC",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -279,8 +399,7 @@ function(x,
 
 setMethod("HQC",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -297,8 +416,7 @@ function(x,
 
 setMethod("MDL2",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -315,8 +433,7 @@ function(x,
 
 setMethod("MDL5",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -333,8 +450,7 @@ function(x,
 
 setMethod("AWE",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -351,8 +467,7 @@ function(x,
 
 setMethod("CLC",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -369,8 +484,7 @@ function(x,
 
 setMethod("ICL",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -387,8 +501,7 @@ function(x,
 
 setMethod("ICLBIC",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -405,8 +518,7 @@ function(x,
 
 setMethod("PRD",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -423,8 +535,7 @@ function(x,
 
 setMethod("SSE",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -441,8 +552,7 @@ function(x,
 
 setMethod("PC",
           signature(x = "ANY"),
-function(x, 
-  pos = 1, ...)
+function(x, pos, ...)
 {
   digits <- getOption("digits"); options(digits = 15)
 
@@ -456,7 +566,3 @@ function(x,
   
   output
 }) ## PC
-
-
-
-
