@@ -141,7 +141,7 @@ function(model,
 {
   digits <- getOption("digits"); options(digits = 15)
   
-  message("RCLSMIX Version 2.8.3")
+  message("RCLSMIX Version 2.8.4")
  
   flush.console()
   
@@ -152,6 +152,8 @@ function(model,
      
   model <- RCLSMIX(model = model, ...)
   
+  model@Zp <- factor(model@Zp, levels = levels(model@Zt))
+  
   model@CM <- table(model@Zt, model@Zp)
   
   model@Accuracy <- sum(diag(model@CM)) / model@ntest
@@ -182,7 +184,7 @@ function(model,
 {
   digits <- getOption("digits"); options(digits = 15)
   
-  message("BFSMIX Version 2.8.3")
+  message("BFSMIX Version 2.8.4")
   
   flush.console()
   
@@ -210,6 +212,8 @@ function(model,
        
       temp <- RCLSMIX(model = temp, ...)
       
+      temp@Zp <- factor(temp@Zp, levels = levels(temp@Zt))
+      
       temp@CM <- table(temp@Zt, temp@Zp)
   
       temp@Accuracy <- sum(diag(temp@CM)) / temp@ntest
@@ -246,6 +250,8 @@ function(model,
        
   model <- RCLSMIX(model = model, ...)
   
+  model@Zp <- factor(model@Zp, levels = levels(model@Zt))
+  
   model@CM <- table(model@Zt, model@Zp)
   
   model@Accuracy <- sum(diag(model@CM)) / model@ntest
